"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveModuleImports = resolveModuleImports;
exports.ResolveSyntaxError = void 0;

var _whatwgUrl = _interopRequireDefault(require("whatwg-url"));

var _resolve = _interopRequireDefault(require("resolve"));

var _pathIsInside = _interopRequireDefault(require("path-is-inside"));

var _path = _interopRequireDefault(require("path"));

var _esModuleLexer = require("./es-module-lexer.js");

var _utils = require("./utils.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-await-in-loop, no-restricted-syntax */
class ResolveSyntaxError extends Error {}
/**
 * @param {string} importPath
 * @param {import('resolve').AsyncOpts} options
 * @returns {Promise<string>}
 */


exports.ResolveSyntaxError = ResolveSyntaxError;

function nodeResolveAsync(importPath, options) {
  return new Promise((resolve, reject) => (0, _resolve.default)(importPath, options, (error, result) => {
    if (error) {
      reject(error);
    } else {
      resolve(result);
    }
  }));
}
/**
 * @typedef {object} ResolveConfig
 * @property {string[]} fileExtensions
 * @property {string[]} moduleDirectories
 * @property {boolean} preserveSymlinks
 */

/**
 * @param {string} rootDir
 * @param {string} sourceFilePath
 * @param {string} importPath
 * @param {ResolveConfig} config
 */


async function resolveImport(rootDir, sourceFilePath, importPath, config) {
  // don't touch url imports
  if (_whatwgUrl.default.parseURL(importPath) !== null) {
    return importPath;
  } // don't touch relative of absolute imports with a file extension


  if ((importPath.startsWith('./') || importPath.startsWith('/')) && _path.default.extname(importPath) !== '') {
    return importPath;
  }

  const sourceFileDir = _path.default.dirname(sourceFilePath);

  try {
    const resolvedImportFilePath = await nodeResolveAsync(importPath, {
      basedir: sourceFileDir,
      extensions: config.fileExtensions,
      moduleDirectory: config.moduleDirectories,
      preserveSymlinks: config.preserveSymlinks,

      packageFilter(packageJson) {
        /* eslint-disable no-param-reassign */
        packageJson.main = packageJson.module || packageJson['jsnext:main'] || packageJson.main;
        return packageJson;
      }

    });

    if (!(0, _pathIsInside.default)(resolvedImportFilePath, rootDir)) {
      throw new Error(`Import "${importPath}" resolved to the file "${resolvedImportFilePath}" which is outside the root directory. ` + 'Install the module locally in the current project, or expand the root directory. If this is a symlink or if you used npm link, ' + ' you can use the preserveSymlinks option');
    }

    const relativeImportFilePath = _path.default.relative(sourceFileDir, resolvedImportFilePath);

    const resolvedImportPath = (0, _utils.toBrowserPath)(relativeImportFilePath);
    return resolvedImportPath.startsWith('.') ? resolvedImportPath : `./${resolvedImportPath}`;
  } catch (error) {
    // make module not found error message shorter
    if (error.code === 'MODULE_NOT_FOUND') {
      throw new Error(`Could not find module "${importPath}".`);
    }

    throw error;
  }
}
/**
 * @param {string} rootDir
 * @param {string} sourceFilePath
 * @param {string} source
 * @param {ResolveConfig} config
 */


async function resolveModuleImports(rootDir, sourceFilePath, source, config) {
  const [imports
  /* exports */
  ,, error] = (0, _esModuleLexer.analyzeModuleSyntax)(source);

  if (error) {
    throw new ResolveSyntaxError(`Syntax error.`);
  }

  let resolvedSource = '';
  let lastIndex = 0;

  for (const imp of imports) {
    const {
      s: start,
      e: end,
      d: dynamicImportIndex
    } = imp;

    if (dynamicImportIndex === -1) {
      // static import
      const importPath = source.substring(start, end);
      const resolvedImportPath = await resolveImport(rootDir, sourceFilePath, importPath, config);
      resolvedSource += `${source.substring(lastIndex, start)}${resolvedImportPath}`;
      lastIndex = end;
    } else if (dynamicImportIndex >= 0) {
      // dynamic import
      const dynamicStart = end + 1;
      const dynamicEnd = dynamicImportIndex - 1;
      const importPath = source.substring(dynamicStart, dynamicEnd);
      const resolvedImportPath = await resolveImport(rootDir, sourceFilePath, importPath, config);
      resolvedSource += `${source.substring(lastIndex, dynamicStart)}${resolvedImportPath}`;
      lastIndex = dynamicEnd;
    }
  }

  if (lastIndex < source.length - 1) {
    resolvedSource += `${source.substring(lastIndex, source.length)}`;
  }

  return resolvedSource;
}